@@ -88,6 +88,8 @@ class view3D():
         self.animate_toolbar = self.plot.app_window.addToolBar('Animate Modeshape')
         self.animate_clear_toolbar = self.plot.app_window.addToolBar('Clear Modeshape')
 
+        # fixed_rotation
+        self.fixed_rotation = None
 
     def add_modeshape(self,dict_shape,run_animation = False,add_note = False):
         """
@@ -399,7 +401,7 @@ class view3D():
         return sphere,vp_actor
 
 
-    def acc_callback(self,point, orientation = None,fixed_rotation = None):
+    def acc_callback(self,point, orientation = None):
         """
         Interactive accelerometer callback function.
 
@@ -407,10 +409,7 @@ class view3D():
         :type point: array(float)
         :param orientation: Orientation in 3D space
         :type orientation: array(float), optional
-        :param fixed_rotation: fixed rotation angle
-        :type fixed_rotation: float
         """
-
         i = int(len(self.all_accs_dynamic)+len(self.all_imps_dynamic)+len(self.all_vps_dynamic))
         size = 10
 
@@ -423,7 +422,7 @@ class view3D():
             rot = r.as_matrix()
 
         self.add_accelerometer(acc)
-        _gg = DynamicPosition(acc, self.plot, i, mesh=self.mesh, size=10,rot = rot,fixed_rotation = fixed_rotation)
+        _gg = DynamicPosition(acc, self.plot, i, mesh=self.mesh, size=10,rot = rot,fixed_rotation = self.fixed_rotation)
         self.plot.add_sphere_widget(_gg.callback, center=_gg.points, color=["k", "r", "g", "b"], radius=10 / 15)
         _gg.translate(point)
         _gg.turn_on = True
@@ -443,20 +442,20 @@ class view3D():
         :param fixed_rotation: fixed rotation angle
         :type fixed_rotation: float
         """
-
+        self.fixed_rotation = fixed_rotation
         self.mesh = mesh
 
         if isinstance(predefined, pd.DataFrame):
             for i, row in predefined.iterrows():
                 point = [row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale]
                 orientation = [row["Orientation_1"], row["Orientation_2"], row["Orientation_3"]]
-                self.acc_callback(point, orientation=orientation,fixed_rotation = fixed_rotation)
+                self.acc_callback(point, orientation=orientation)
 
         self.plot.enable_point_picking(callback=self.acc_callback, color="r", show_message="", show_point=False)
         self.plot.add_text("Press P too add an accelerometer (hold down letter T to disable snapping to mesh).",
                            font_size=10, color="k", font="times", name="text")
 
-    def imp_callback(self,point, direction = None,fixed_rotation = None):
+    def imp_callback(self,point, direction = None):
         """
         Interactive impact callback function.
 
@@ -464,8 +463,6 @@ class view3D():
         :type point: array(float)
         :param orientation: Orientation in 3D space
         :type orientation: array(float)
-        :param fixed_rotation: fixed rotation angle
-        :type fixed_rotation: float
         """
 
         i = int(len(self.all_accs_dynamic)+len(self.all_imps_dynamic)+len(self.all_vps_dynamic))
@@ -479,7 +476,7 @@ class view3D():
             #rot = np.diag([1]*3)
             rot = rotation_matrix_from_vectors(direction,[0, 0, 1]).T
 
-        _gg = DynamicPosition([imp], self.plot, i, mesh=self.mesh, size=10,rot = rot,snap_outward = False, fixed_rotation = fixed_rotation)
+        _gg = DynamicPosition([imp], self.plot, i, mesh=self.mesh, size=10,rot = rot,snap_outward = False, fixed_rotation = self.fixed_rotation)
         self.plot.add_sphere_widget(_gg.callback, center=_gg.points, color=["k", "r", "g", "b"], radius=10 / 15)
         _gg.translate(point)
         _gg.turn_on = True
@@ -499,26 +496,24 @@ class view3D():
         :param fixed_rotation: fixed rotation angle
         :type fixed_rotation: float
         """
-
+        self.fixed_rotation = fixed_rotation
         self.mesh = mesh
 
         if isinstance(predefined, pd.DataFrame):
             for i, row in predefined.iterrows():
                 point = [row["Position_1"] * scale, row["Position_2"] * scale, row["Position_3"] * scale]
                 direction = [row["Direction_1"], row["Direction_2"], row["Direction_3"]]
-                self.imp_callback(point, direction=direction,fixed_rotation = fixed_rotation)
+                self.imp_callback(point, direction=direction)
 
         self.plot.enable_point_picking(callback=self.imp_callback, color="r", show_message="", show_point=False)
         self.plot.add_text("Press P too add an impact (hold down letter T to disable snapping to mesh).", font_size = 10,color = "k",font  = "times",name = "text")
 
-    def vp_callback(self,point,fixed_rotation = None):
+    def vp_callback(self,point):
         """
         Interactive virtual point callback function.
 
         :param point: Point in 3D space
         :type point: array(float)
-        :param fixed_rotation: fixed rotation angle
-        :type fixed_rotation: float
         """
 
         i = int(len(self.all_accs_dynamic)+len(self.all_imps_dynamic)+len(self.all_vps_dynamic))
@@ -526,7 +521,7 @@ class view3D():
 
         acc, _ = self.add_vp([size/2, size/2, size/2], size=size, opacity=.1)
 
-        _gg = DynamicPosition([acc], self.plot, i, mesh=self.mesh, size=4, snap_outward=False,fixed_rotation = fixed_rotation)
+        _gg = DynamicPosition([acc], self.plot, i, mesh=self.mesh, size=4, snap_outward=False,fixed_rotation = self.fixed_rotation)
         self.plot.add_sphere_widget(_gg.callback, center=_gg.points, color=["k", "r", "g", "b"], radius=4 / 15)
 
         _gg.turn_on = True
@@ -985,6 +980,8 @@ class DynamicPosition():
 
     def __init__(self, objects, p, N, mesh=None, snap_outward=True, size=1, rot = np.diag([1]*3), fixed_rotation = None):
         # set size and static points
+
+
         self.size = size
         self.points = np.array([[size / 2, size / 2, size / 2],
                                 [0, size / 2, size / 2],
@@ -1207,13 +1204,14 @@ class DynamicPosition():
                 _vec2 = (np.asarray(self.local_widgets[i, :]))
 
                 # define the rotational matrix based on angle of rotation
-                theta = angle(_vec1, _vec2)
+                theta = -1*angle(_vec1, _vec2)
 
                 # overwrite calculated rotational angle
                 if self.fixed_theta != None:
                     theta =  self.fixed_theta*(np.pi/180)
 
-                rot = M(self.local_orientation[i - 1, :], theta)
+                rot = M(_vec2, theta)
+                #rot = rotation_matrix_from_vectors(_vec1, _vec2)
 
                 # rotate everything within accelerometer
                 for item in self.objects:
